package org.siqisource.agave.orm.condition;

import java.util.List;

import org.siqisource.agave.orm.Condition;
import org.siqisource.agave.orm.SqlCompareExpression;
import org.siqisource.agave.orm.expression.BetweenValueExpression;
import org.siqisource.agave.orm.expression.ListValueExpression;
import org.siqisource.agave.orm.expression.NoValueExpression;
import org.siqisource.agave.orm.expression.SingleValueExpression;
import org.siqisource.agave.orm.util.NameConverter;

public class SimpleCondition extends Condition {

	public static final String AND_PT = " AND ( ";

	public static final String AND = " AND ";

	public static final String OR_PT = " OR ( ";

	public static final String OR = " OR ";

	public static final String END_PT = " ) ";

	/**
	 * if className.propertyName it will ben classname.property_name
	 * 
	 * @param property
	 * @return
	 */
	private String getColumn(String property) {
		String tableAlias = "";
		if (property.indexOf(".") != -1) {
			String[] parts = property.split("[.]");
			tableAlias = parts[0].toLowerCase() + ".";
			property = parts[1];
		}
		
		return tableAlias + NameConverter.propertyToColumn(property);
	}

	public void andIsNotNull(String propertyName) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new NoValueExpression(AND, column,
				" IS NOT NULL ", "");
		this.addExpression(expression);
	}

	public void andIsNull(String propertyName) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new NoValueExpression(AND, column,
				" IS NULL ", "");
		this.addExpression(expression);
	}

	public void andEqual(String propertyName, Object value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " = ", value, "");
		this.addExpression(expression);
	}

	public void andLike(String propertyName, String value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " LIKE ", "%" + value + "%", "");
		this.addExpression(expression);
	}
	
	public void andNotLike(String propertyName, String value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " NOT LIKE ", "%" + value + "%", "");
		this.addExpression(expression);
	}

	public void andBeginWith(String propertyName, String value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " LIKE ", value + "%", "");
		this.addExpression(expression);
	}

	public void andEndWith(String propertyName, String value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " LIKE ", "%" + value, "");
		this.addExpression(expression);
	}

	public void andNotEqual(String propertyName, Object value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " <> ", value, "");
		this.addExpression(expression);
	}

	public void andGreater(String propertyName, Object value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " > ", value, "");
		this.addExpression(expression);
	}

	public void andNoLess(String propertyName, Object value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " >= ", value, "");
		this.addExpression(expression);
	}

	public void andLess(String propertyName, Object value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " < ", value, "");
		this.addExpression(expression);
	}

	public void andNoGreater(String propertyName, Object value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new SingleValueExpression(AND,
				column, " <= ", value, "");
		this.addExpression(expression);
	}

	public void andBetween(String propertyName, Object beginValue,
			Object endValue) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new BetweenValueExpression(AND,
				column, beginValue, endValue, "");
		this.addExpression(expression);
	}

	public void andIn(String propertyName, List<Object> value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new ListValueExpression(AND, column,
				" IN ", value, "");
		this.addExpression(expression);
	}

	public void andIn(String propertyName, String value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new ListValueExpression(AND, column,
				" IN ", value, "");
		this.addExpression(expression);
	}

	public void andNotIn(String propertyName, List<Object> value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new ListValueExpression(AND, column,
				" NOT IN ", value, "");
		this.addExpression(expression);
	}

	public void andNotIn(String propertyName, String value) {
		String column = getColumn(propertyName);
		SqlCompareExpression expression = new ListValueExpression(AND, column,
				" NOT IN ", value, "");
		this.addExpression(expression);
	}

}
